#include <mwm5.h>

#include <Arduino.h>
#include <M5Stack.h>

// ①パーサーの準備
static AsciiParser parse_ascii(256);

void setup() {
  // M5Stackの初期化
  M5.begin(true, false, true, false); 

  // ②シリアルポートの初期化
  Serial2.setRxBufferSize(512);
  Serial2.begin(115200, SERIAL_8N1, 16, 17);
}

void loop() {
  M5.update();

  // ③シリアル受信
  while (Serial2.available()) {
    int c = char_t(Serial2.read());
    parse_ascii << c;

    if (parse_ascii) {
      // ④センサ値の取得
      // TWELITEパケットの取得
      auto && pkt = newTwePacket(parse_ascii.get_payload());
      // TWELITE PALかどうかの判定
      if (identify_packet_type(pkt) == E_PKT::PKT_PAL) {
        // TWELITE PALであれば、PALの型に変換
        TwePacketPal& pal = refTwePacketPal(pkt);
        // 以下palのプロパティを使って、各種データを取得できる

        // ⑤PALの種別判定
        switch (pal.u8palpcb) {
          case E_PAL_PCB::MAG: {
            // 開閉センサーパルのとき
            // ⑥センサの値の取得
            PalMag mag = pal.get_PalMag();
            if (mag.u8MagStat == 0) {
              // 開いている
              M5.Lcd.fillScreen(RED);
              M5.Lcd.print("OPEN\n");
            } else {
              // 閉じている
              M5.Lcd.fillScreen(GREEN);
              M5.Lcd.print("CLOSE\n");
            }
            break;
          }
          case E_PAL_PCB::AMB: {
            // 環境センサーパルのとき
            // ⑥センサの値の取得
            PalAmb amb = pal.get_PalAmb();
            M5.Lcd.printf("Temperature : %f\n", 
               (double)amb.i16Temp / 100.0);
            M5.Lcd.printf("Humidity : %f\n",
                (double)(amb.u16Humd + 50) / 100.0);
            M5.Lcd.printf("Illumination : %d\n", amb.u32Lumi);
            break;
          }
          case E_PAL_PCB::MOT: {
            // 動作センサーパルのとき
            // ⑥センサの値の取得
            PalMot mot = pal.get_PalMot();
            M5.Lcd.printf("X=%d, Y=%d, Z=%d\n", 
               mot.i16X[0], mot.i16Y[0], mot.i16Z[0]);
            break;            
          }
        }
      }
    }
  }
}
