#include <mwm5.h>

#include <Arduino.h>
#include <M5Stack.h>

static AsciiParser parse_ascii(256);

// 開閉センサーパル、環境センサーパル、動作センサーパルの、それぞれのターミナル
static TWETerm_M5_Console scrMag(40, 5, { 0, 0, 320, 5 * 16}, M5);
static TWETerm_M5_Console scrAmb(40, 5, { 0, 5 * 16, 320, 5 * 16}, M5);
static TWETerm_M5_Console scrMot(40, 5, { 0, 10 * 16, 320, 5 * 16}, M5);

void setup() {
  // M5Stackの初期化
  M5.begin(true, false, true, false); 

  // シリアルポートの初期化
  Serial2.setRxBufferSize(512);
  Serial2.begin(115200, SERIAL_8N1, 16, 17);
  
  // ターミナルの初期化
  // フォントの作成
  // 東雲フォント16ピクセル
  TWEFONT::createFontShinonome16(10, 0, 0);
  // 倍角
  TWEFONT::createFontShinonome16(11, 0, 0, 
    TWEFONT::U32_OPT_FONT_TATEBAI | TWEFONT::U32_OPT_FONT_YOKOBAI); 

  scrMag.set_font(11);
  scrAmb.set_font(10);
  scrMot.set_font(10);
 
  // ③前景色と背景色の設定
  scrMag.set_color(ALMOST_WHITE, color565(0, 0, 0));
  scrAmb.set_color(ALMOST_WHITE, color565(0, 150, 0));
  scrMot.set_color(ALMOST_WHITE, color565(150, 0, 0));
}

void loop() {
  M5.update();

  // シリアル受信
  while (Serial2.available()) {
    int c = char_t(Serial2.read());
    parse_ascii << c;

    if (parse_ascii) {
      // パース完了
      // TWELITEパケットの取得
      auto && pkt = newTwePacket(parse_ascii.get_payload());
      // TWELITE PALかどうかの判定
      if (identify_packet_type(pkt) == E_PKT::PKT_PAL) {
        // TWELITE PALであれば、PALの型に変換
        TwePacketPal& pal = refTwePacketPal(pkt);
        // ここでpalのプロパティを使って、各種データを取得できる

        // PALの種別判定
        switch (pal.u8palpcb) {
          case E_PAL_PCB::MAG: {
            // 開閉センサーパルのとき温度
            PalMag mag = pal.get_PalMag();
            if (mag.u8MagStat == 0) {
              // 開いている
              scrMag << TermAttr(TERM_COLOR_FG_RED);
              scrMag << "開";              
            } else {
              // 閉じている
              scrMag << TermAttr(TERM_COLOR_FG_GREEN);
              scrMag << "閉";              
            }
            break;
          }
          case E_PAL_PCB::AMB: {
            // 環境センサーパルのとき
            PalAmb amb = pal.get_PalAmb();	
            scrAmb << printfmt("温度  %f℃\n",
               (double)amb.i16Temp / 100.0);
            scrAmb << printfmt("湿度  %f％\n",
                (double)(amb.u16Humd + 50) / 100.0);
            scrAmb << printfmt("照度  %dルクス",  amb.u32Lumi);
            break;
          }
          case E_PAL_PCB::MOT: {
            // 動作センサーパルのとき
            PalMot mot = pal.get_PalMot();
            scrMot << printfmt("X=%d\nY=%d\nZ=%d\n", 
               mot.i16X[0], mot.i16Y[0], mot.i16Z[0]);
            break;            
          }
        }
      }
    }
  }

  // ターミナルのリフレッシュ
  scrMag.refresh();
  scrAmb.refresh();
  scrMot.refresh();
}
