# モジュールのインポート
from wsgiref.simple_server import make_server

# Jinja2
from jinja2 import Template, Environment, FileSystemLoader

# AppPALライブラリ
import sys
sys.path.append('./MNLib/')
from apppal import AppPAL

# マルチプロセス
from multiprocessing import Value, Process

# COMポート
port = '/dev/ttyUSB0'

# マルチプロセス処理でセンサーデータを取得する
def sensing(t, h, i):
  PAL = AppPAL(port = port)
  while True:
    if PAL.ReadSensorData():
      data = PAL.GetDataDict()
      t.value = data['Temperature']
      h.value = data['Humidity']
      i.value = data['Illuminance']

# 共有メモリを作る
temperature = Value('d')
humidity = Value('d')
illuminance = Value('i')

# プロセスとして実行
process1 = Process(target=sensing, args=[temperature, humidity, illuminance])
process1.start()

# 受信処理
def process(env, response):
  global temperature, humedity, illuminance
  # （1）ヘッダを返す
  headers = [
    ('Content-Type', 'text/html; charset=utf-8')
  ]
  response('200 OK', headers)
  # （2）コンテンツを返す
  # Jinja2のテンプレートを読み込む
  env = Environment(loader=FileSystemLoader(''))
  template = env.get_template('index.tpl')

  # 値を差し込んでHTMLにする
  html = template.render({
    'Temperature' : temperature.value,
    'Humidity' : humidity.value,
    'Illuminance' : illuminance.value
  })
  return [html.encode('utf-8')]

# サーバーの立ち上げ
with make_server('', 80, process) as httpd:
  print("started")
  httpd.serve_forever()
