import boto3
import json
import decimal
from boto3.dynamodb.conditions import Key

print('Loading function')
dynamo = boto3.client('dynamodb')


def respond(err, res=None):
    return {
        'statusCode': '400' if err else '200',
        'body': err.message if err else json.dumps(res),
        'headers': {
            'Content-Type': 'application/json',
        },
    }

def lambda_handler(event, context):
    operation = event['httpMethod']
    if operation == 'GET':
        topic = None
        starttime=None
        endtime = None
        if event.get('queryStringParameters'):
            topic = event["queryStringParameters"].get("topic")
            starttime = event["queryStringParameters"].get("start")        
            endtime = event["queryStringParameters"].get("end") 

        if not topic:
            topic = 'pal/amb0001'
        if not starttime:
            starttime = 0
        if not endtime:
            endtime = 9999999999999
        
        starttime = decimal.Decimal(starttime)
        endtime = decimal.Decimal(endtime)
        
        dynamodb = boto3.resource('dynamodb')
        table = dynamodb.Table('sensortable')

        data = []
        response = {}
        response['LastEvaluatedKey'] = None
        while 'LastEvaluatedKey' in response:
            if response['LastEvaluatedKey']:
                response = table.query(
                    KeyConditionExpression=Key('topic').eq(topic) & Key('timestamp').between(starttime, endtime),                     
                    ExclusiveStartKey=response['LastEvaluatedKey']
                )        
            else:
                response = table.query(
                    KeyConditionExpression=Key('topic').eq(topic) & Key('timestamp').between(starttime, endtime),                     
                )        
                
            data += response['Items']        

        csv = ""
        csv += ",".join(['topic', 'timestamp', 'temperature', 'humidity', 'illuminance']) + "\n" 
        
        for item in data:
            r = [item['topic'], str(item['timestamp']),
                str(item['data']['temperature']),
                str(item['data']['humidity']),
                str(item['data']['illuminance']),
            ]
            csv += ','.join(r) + "\n"

        return {
            'statusCode': '200',
            'body': csv,
            'headers': {
                'Content-Type': 'text/csv',
            },
        }

    return respond(ValueError('Unsupported method "{}"'.format(operation)))
